/**
 * Proiect PM - Connect 4
 *	Corut Fabrizio
 *  Grupa : 332CC
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "utils.h"

#define INIT_ROW 0
#define INIT_COL 0

volatile int timer_row = -1;
volatile int crt_col = 0;
volatile int matrix[8][8];
volatile int aux_matrix[8][8];
int turn, state, game_state;
char c;

/* intreruperea */
ISR(TIMER1_CAPT_vect)
{	
	int i;
	timer_row++;
	timer_row = timer_row % 8;
	PORTA = 56 + timer_row;
	for (i = 0; i < 8; ++i)
		if (matrix[timer_row][i] == 1) {
			if (aux_matrix[timer_row][i] == 1) {
				PORTB |= (1 << i);
				_delay_us(20);
				PORTB &= ~(1 << i);
			}
			else {
				PORTC |= (1 << i);
				_delay_us(20);
				PORTC &= ~(1 << i);
			}
		}
}

void timer1_init()
{
	ICR1 = 320;
    /* activeaza intreruperea de comparare cu ICR1 = TOP */
    TIMSK1 = (1 << ICIE1);
    
    /* modul CTC cu ICR1 */
    /* prescaler 1024 */
    TCCR1B = (2 << CS10) | (3 << WGM12);
}

/* initializeaza porturile */
void initialize_game()
{
	int i, j;	

	for (i = 0; i < 6; i++) {
		if (i < 3) {
			/* BCD */
			DDRA |= (1 << i);
		}
		else {
			/* butoanele ca intrari */
            DDRA &= ~(1 << i);
			/* rezistentele de pull-up */
			PORTA |= (1 << i);
		}
	}
    /* ledurile ca si iesiri */
	for (j = 0; j < 8; j++) {
		DDRB |= (1 << j);
		DDRC |= (1 << j);
	}

}

/* caderea discului */
void drop_disk(int row, int col)
{
	int i;

	for (i = 0; i < row - 1; i++) {
		matrix[i][col] = 1;
		aux_matrix[i][col] = 1;
		_delay_ms(70);
		matrix[i][col] = 0;
		aux_matrix[i][col] = 0;
	}
	
	matrix[i][col] = 1;
	aux_matrix[i][col] = 1;
}

/* returneaza linia pe care se pune discul */
int disk_row(int col)
{
    int i;
    
    for (i = 2;  i < 8; i++)
        if (matrix[i][col] == 1) {
            return i--;
        }
    
    return 8;
}

void display_win()
{
    int i;
    
    for (i = 2; i < 6; i++) {
        matrix[0][i] = 1;
        aux_matrix[0][i] = 1;
    }
    
    matrix[1][1] = 1;
    aux_matrix[1][1] = 1;
    matrix[1][6] = 1;
    aux_matrix[1][6] = 1;
    
    matrix[2][0] = 1;
    aux_matrix[2][0] = 1;
    matrix[2][2] = 1;
    aux_matrix[2][2] = 1;
    matrix[2][5] = 1;
    aux_matrix[2][5] = 1;
    matrix[2][7] = 1;
    aux_matrix[2][7] = 1;
    
    matrix[3][0] = 1;
    aux_matrix[3][0] = 1;
    matrix[3][7] = 1;
    aux_matrix[3][7] = 1;
    
    matrix[4][0] = 1;
    aux_matrix[4][0] = 1;
    matrix[4][1] = 1;
    aux_matrix[4][1] = 1;
    matrix[4][6] = 1;
    aux_matrix[4][6] = 1;
    matrix[4][7] = 1;
    aux_matrix[4][7] = 1;
    
    matrix[5][0] = 1;
    aux_matrix[5][0] = 1;
    matrix[5][2] = 1;
    aux_matrix[5][2] = 1;
    matrix[5][3] = 1;
    aux_matrix[5][3] = 1;
    matrix[5][4] = 1;
    aux_matrix[5][4] = 1;
    matrix[5][5] = 1;
    aux_matrix[5][5] = 1;
    matrix[5][7] = 1;
    aux_matrix[5][7] = 1;
    
    matrix[6][1] = 1;
    aux_matrix[6][1] = 1;
    matrix[6][6] = 1;
    aux_matrix[6][6] = 1;
    
    for (i = 2; i < 6; i++) {
        matrix[7][i] = 1;
        aux_matrix[7][i] = 1;
    }
}

void play_game()
{
	matrix[0][0] = 1;
	crt_col = INIT_COL;

	while(1) {
        if ((PINA & (1 << PA5)) == 0) {
            if (crt_col < 7) {
                matrix[0][crt_col] = 0;
                aux_matrix[0][crt_col] = 0;
                crt_col++;
                matrix[0][crt_col] = 1;
                aux_matrix[0][crt_col] = 1;
            }
        } else if ((PINA & (1 << PA3)) == 0) {
            if (crt_col > 0) {
                matrix[0][crt_col] = 0;
                aux_matrix[0][crt_col] = 0;
                crt_col--;
                matrix[0][crt_col] = 1;
                aux_matrix[0][crt_col] = 1;
            }
        } else if ((PINA & (1 << PA4)) == 0) {
            if (matrix[1][crt_col] != 1) {
                int curr_row = disk_row(crt_col);
                drop_disk(curr_row, crt_col);
                crt_col = INIT_COL;
                matrix[INIT_ROW][crt_col] = 1;
                
                if (check_win()) {
                    display_win();
                    return;
                }
            }
        }
        _delay_ms(100);
	}
}

int main(void) {
	initialize_game();
	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);
    
    sei();
    timer1_init();
    play_game();
	return 0;
}


/* victorie pe coloana */
int check_col(int row, int col)
{
	int i;

	if (row > 4)
		return 0;

	for (i = row; i < row + 4; i++)
		if (aux_matrix[i][col] == 0)
			return 0;

	return 1;
}

/* victorie pe linie */
int check_row(int row, int col)
{
	int i;

	if (col > 4)
		return 0;

	for (i = col; i < col + 4; i++)
		if (aux_matrix[row][i] == 0)
			return 0;

	return 1;
}

/* victorie pe diagonala principala */
int check_first_diag(int row, int col)
{
	int i;

	if (row > 4 || col > 4)
		return 0;

	for (i = 0; i < 4; i++)
		if (aux_matrix[row + i][col + i] == 0)
			return 0;

	return 1;
}

/* victoarie pe diagonala secundara */
int check_second_diag(int row, int col)
{
	int i;

	if (row > 4 || col < 3)
		return 0;

	for (i = 0; i < 4; i++)
		if (aux_matrix[row + i][col - i] == 0)
			return 0;

	return 1;
}

/* victorie pentru o matrice */
int check_win()
{
	int i, j;

	for (i = 1; i < 8; i++)
		for (j = 0; j < 8; j++) {
			if (check_col(i, j) == 1)
				return 1;
			else if (check_row(i, j) == 1)
				return 1;
			else if (check_first_diag(i, j) == 1)
				return 1;
			else if (check_second_diag(i, j) == 1)
				return 1;
		}

	return 0;
}

